> class Neuron_Networks:

N代表单个样本维度，M代表样本个数  

- __init__:

layers:每层神经元个数，整数构成的list   
act_func:每层激活函数，类构成的list   
weight_define:w初始化方法，类   

- forward:   
data:输入数据，N*M   
返回值：前向计算每层的a,z，都是字典   
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
a, z = net.forward(np.array([[1,2],
                             [3,4],
                             [5,6]]))
a, z
```

- backward:  
a:每层神经元激活函数输出   
y:one-hot标签，类别数*M   
z:每层神经元净输出   
返回值：每层神经元误差敏感度delta,字典  
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import LossFunctions
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
a, z = net.forward(np.array([[1,2],
                             [3,4],
                             [5,6]]))
y = np.array([[0,1],
              [1,0]])
loss_f = LossFunctions.MSE()
delta = net.backward(a ,y, z, loss_f)
delta
```
- compute_gradient:    
a:每层神经元激活函数输出   
delta:每层神经元误差敏感度   
返回值：每两层神经元之间w的梯度  
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import LossFunctions
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
a, z = net.forward(np.array([[1,2],
                             [3,4],
                             [5,6]]))
y = np.array([[0,1],
              [1,0]])
loss_f = LossFunctions.MSE()
delta = net.backward(a ,y, z, loss_f)
w_grad = net.compute_gradient(a, delta)
w_grad
```

- gradient_descent:   
w_grad:每两层神经元间w的梯度   
alpha:学习率   
无返回值，自动更新网络的连接权值w   
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
a, z = net.forward(np.array([[1,2],
                             [3,4],
                             [5,6]]))
w_grad = {}
w_grad[1] = np.array([[1,2,3],
                      [4,5,6]])
net.gradient_descent(w_grad, 0.1)
net.w
```

- TrainBP:  
train_data:训练数据，N * M  
train_label:训练数据标签，类别数*M  
batch_size:每批样本个数  
alpha:学习率  
num_epoch:迭代次数  
validate_ratio:验证集占训练集百分比  
show_ratio:准确率展示间隔，每show_ratio * num_epoch次迭代展示一次准确率  
loss_func:性能函数，类  
测试代码：
```python
import Model
import ActiveFunctions as Actfunc
import LossFunctions as LossFunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
label = np.array([[1,0,1],
                  [0,1,0]])
loss_f = LossFunc.MSE()
net.TrainBP(data, label, loss_func = loss_f)
```
- Test:  
test_data:测试数据，N * M  
test_label:测试数据标签，类别数 * M  
输出准确率，保留四位小数  
测试代码：
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
label = np.array([[1,0,1],
                  [0,1,0]])
net.Test(data, label)
```

- Predict:  
data:待预测数据，N * M  
返回值:每个样本所属类别,长度为M的array  
测试代码：
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
net.Predict(data)
```
- Correct:  
x:预测结果，类别数 * M  
y:真实结果，类别数 * M  
返回值:预测正确的样本个数  
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
label = np.array([[1,0,1],
                  [0,1,0]])
net.Correct(data, label)
```

- Accuracy:  
x:预测结果，每列一个one-hot，类别数 * M  
y:真实结果，每列一个one-hot，类别数 * M  
返回值:准确率  
测试代码：
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
label = np.array([[1,0,1],
                  [0,1,0]])
net.Accuracy(data, label)
```

- Normalization:

data:待标准化数据，N * M  
返回值:标准化后数据  
测试代码：
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
data = net.Normalization(data)
data
```
<br>
<br>

> class Hebb:

- __init__:  
l1:输入层神经元个数  
l2:输出层神经元个数  
act_func:激活函数  
测试代码：  
```python
import Model
import ActiveFunctions
hardlim = ActiveFunctions.Hardlim()
net = Model.Hebb(4,4,hardlim)
```

- Train:  
data:训练数据，input_dim * M  
data_labels:训练数据对应输出 output_dim * M  
alpha:学习率  
num_epoch:迭代次数  
测试代码：  
```python
import Model
import ActiveFunctions
import numpy as np
hardlims = ActiveFunctions.Hardlims()
net = Model.Hebb(4,4,hardlims)
data = np.array([[1,-1],
                 [1,1],
                 [-1,-1],
                 [-1,1],])
labels = np.array([[1,-1],
                 [1,1],
                 [-1,-1],
                 [-1,1],])
net.Train(data, labels, 0.01)
```
- Predict:  
data:待预测数据，input_dim * M  
返回值：output_dim*M的array，每列为一个样本的预测结果  
测试代码：  
```python
import Model
import ActiveFunctions
import numpy as np
hardlims = ActiveFunctions.Hardlims()
net = Model.Hebb(4,4,hardlims)
data = np.array([[1,-1],
                 [1,1],
                 [-1,-1],
                 [-1,1],])
net.Predict(data)
```
<br>
<br>

> class Perceptron: 

- __init__:   
l1:输入层神经元个数    
l2:输出层神经元个数  
act_func:激活函数  
bias:偏置值b     
测试代码：
```python
import Model
import ActiveFunctions
hardlim = ActiveFunctions.Hardlim()
net = Model.Perceptron(3, 1, hardlim)
```

- Normalization:   
data:待标准化数据，N * M  
返回值:标准化后数据  
测试代码：  
```python
import Model
import ActiveFunctions as Actfunc
import numpy as np
act_f = Actfunc.Linear()
net = Model.Neuron_Networks([3,2],[act_f])
data = np.array([[0,1,2],
                 [3,4,5],
                 [6,7,8]])
data = net.Normalization(data)
data
```

- Train:  
train_data:训练数据，input_dim * M  
train_label:训练数据所对输出，output_dim * M  
alpha:学习率  
num_epoch:迭代次数  
测试代码：  
```python
import Model
import ActiveFunctions
import numpy as np
hardlims = ActiveFunctions.Hardlims()
net = Model.Perceptron(2,1,hardlims)
data = np.array([[0,0,1,1],
                 [0,1,0,1]])
labels = np.array([[0,1,1,0]])
net.Train(data, labels)
```

- Predict:  
data:待预测数据，input_dim * M   
返回值：预测结果，output_dim * M的array  
测试代码：   
```python
import Model
import ActiveFunctions
import numpy as np
hardlims = ActiveFunctions.Hardlims()
net = Model.Perceptron(2,1,hardlims)
data = np.array([[0,0,1,1],
                 [0,1,0,1]])
net.Predict(data)
```
